@@ -1,5 +1,5 @@
 import asyncio
-from concurrent.futures._base import Executor
+from concurrent.futures import Executor
 from functools import partial, total_ordering
 from pathlib import Path
 from typing import Any, Awaitable, Callable, Generator, TypeVar, Union
@@ -111,7 +111,7 @@ class AsyncFileIOBase:
 
     async def __aexit__(
         self, exc_type: Any,
-        exc_val: Any, exc_tb: Any
+        exc_val: Any, exc_tb: Any,
     ) -> None:
         if self.fp is None:
             raise RuntimeError("file is not opened")
