@@ -1,4 +1,5 @@
 from .base import Service, ServiceMeta, SimpleServer
+from .process import ProcessService, RespawningProcessService
 from .profiler import Profiler
 from .tcp import TCPServer
 from .tls import TLSServer
@@ -8,7 +9,9 @@ from .udp import UDPServer
 
 __all__ = (
     "MemoryTracer",
+    "ProcessService",
     "Profiler",
+    "RespawningProcessService",
     "Service",
     "ServiceMeta",
     "SimpleServer",
